//=======================================================================================
// ____          ____    __    ______     __________   __      __       __        __         
// \    \       |    |  |  |  |   _   \  |___    ___| |  |    |  |     /  \      |  |        
//  \    \      |    |  |  |  |  |_)   |     |  |     |  |    |  |    /    \     |  |        
//   \    \     |    |  |  |  |   _   /      |  |     |  |    |  |   /  /\  \    |  |        
//    \    \    |    |  |  |  |  | \  \      |  |     |   \__/   |  /  ____  \   |  |____    
//     \    \   |    |  |__|  |__|  \__\     |__|      \________/  /__/    \__\  |_______|   
//      \    \  |    |   ________________________________________________________________    
//       \    \ |    |  |  ______________________________________________________________|   
//        \    \|    |  |  |         __          __     __     __     ______      _______    
//         \         |  |  |_____   |  |        |  |   |  |   |  |   |   _  \    /  _____)   
//          \        |  |   _____|  |  |        |  |   |  |   |  |   |  | \  \   \_______    
//           \       |  |  |        |  |_____   |   \_/   |   |  |   |  |_/  /    _____  \   
//            \ _____|  |__|        |________|   \_______/    |__|   |______/    (_______/   
//
//  This file is part of VirtualFluids. VirtualFluids is free software: you can 
//  redistribute it and/or modify it under the terms of the GNU General Public
//  License as published by the Free Software Foundation, either version 3 of 
//  the License, or (at your option) any later version.
//  
//  VirtualFluids is distributed in the hope that it will be useful, but WITHOUT 
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
//  for more details.
//  
//  You should have received a copy of the GNU General Public License along
//  with VirtualFluids (see COPYING.txt). If not, see <http://www.gnu.org/licenses/>.
//
//! \file DataBaseStruct.h
//! \ingroup DataBase
//! \author Stephan Lenz
//=======================================================================================
#ifndef DataBaseStruct_H
#define DataBaseStruct_H

#include "Core/DataTypes.h"

#include <VirtualFluidsDefinitions.h>

//! Simple struct that carries variables for kernel execution
struct VF_PUBLIC DataBaseStruct
{
    uint  numberOfCells;
    uint  numberOfFaces;
    
    uint  numberOfCoarseGhostCells;
    uint  numberOfFineGhostCells;

    uint* cellToCell;
    uint* faceToCell;

    uint* parentCell;

    uint* fineToCoarse;
    uint* coarseToFine;

    real* faceCenter;
    real* cellCenter;

    CellProperties* cellProperties;

    char* faceOrientation;

    real*            data;
    realAccumulator* dataUpdate;

    real* massFlux;

    real* diffusivity;

    int* crashCellIndex;
};


#endif